

/*
 *   Copyright 2012 Andrea Vinci <uomonero84@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "nvidiaengine.h"

#include <Plasma/DataContainer>
#include <QProcess>

#include <QXmlQuery>


#include <QIODevice>
#include <QBuffer>



NvidiaEngine::NvidiaEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent, args)
{
    // We ignore any arguments - data engines do not have much use for them
    Q_UNUSED(args)
    gpu0="gpu0";
    setMinimumPollingInterval(333);
}
void NvidiaEngine::init(){
    updateSourceEvent(gpu0);
}
bool NvidiaEngine::sourceRequestEvent(const QString &name)
{
    return updateSourceEvent(name);
}

bool NvidiaEngine::updateSourceEvent(const QString &name)
{
  if(name.compare(gpu0)==0){
    QProcess nvsmi;
    nvsmi.start("nvidia-smi -q -x");
    nvsmi.waitForFinished(-1);
    QString nvsmi_stdout=nvsmi.readAllStandardOutput();

    
    QBuffer device;
    device.setData(nvsmi_stdout.toUtf8());
    device.open(QIODevice::ReadOnly);
    QXmlQuery query;
    query.bindVariable("inputDocument", &device);
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/gpu/temperature/gpu_temp/string()");

    if(!query.isValid())return false;
    QStringList result0;
    if(!query.evaluateTo(&result0))return false;

    QStringListIterator it0(result0);
    if (it0.hasNext()){
      QString temp= it0.next();
      setData(name,I18N_NOOP("gpu_temp"),temp);
      
    }
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/gpu/product_name/string()");
    QStringList result1;
    if(!query.evaluateTo(&result1))return false;
    QStringListIterator it1(result1);
    if (it1.hasNext()){
      QString product_name= it1.next();
      setData(name,I18N_NOOP("product_name"),product_name);
      
    }
    
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/driver_version/string()");
    QStringList result2;
    if(!query.evaluateTo(&result2))return false;
    QStringListIterator it2(result2);
    if (it2.hasNext()){
      QString driver_version= it2.next();
      setData(name,I18N_NOOP("driver_version"),driver_version);
      
    }
    
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/gpu/memory_usage/total/string()");
    QStringList result3;
    if(!query.evaluateTo(&result3))return false;
    QStringListIterator it3(result3);
    if (it3.hasNext()){
      QString memory_total= it3.next();
      setData(name,I18N_NOOP("memory_total"),memory_total);
      
    }
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/gpu/memory_usage/used/string()");
    QStringList result4;
    if(!query.evaluateTo(&result4))return false;
    QStringListIterator it4(result4);
    if (it4.hasNext()){
      QString memory_used= it4.next();
      setData(name,I18N_NOOP("memory_used"),memory_used);
      
    }
    query.setQuery("declare variable $inputDocument external;"
                         "doc($inputDocument)//nvidia_smi_log/gpu/memory_usage/free/string()");
    QStringList result5;
    if(!query.evaluateTo(&result5))return false;
    QStringListIterator it5(result5);
    if (it5.hasNext()){
      QString memory_free= it5.next();
      setData(name,I18N_NOOP("memory_free"),memory_free);
    }
    
    
    return true;

    
  }
  return false;
}

K_EXPORT_PLASMA_DATAENGINE(nvidiaDeviceMonitorEngine, NvidiaEngine)

#include "nvidiaengine.moc"
